# -*- coding: utf-8 -*-
import base64

from collections import namedtuple
from datetime import datetime
import logging

from odoo import models, fields, api, SUPERUSER_ID
from odoo.exceptions import UserError
from odoo.tools.parse_version import parse_version

OdooRelease = namedtuple('OdooRelease', 'version date')

RELEASES = [
    OdooRelease('9.0+e', '20/06/2015'),
    OdooRelease('9.0e', '20/06/2015'),
    OdooRelease('9.saas~7e', '20/08/2015'),
    OdooRelease('9.saas~8e', '20/10/2015'),
    OdooRelease('9.saas~9e', '20/12/2015'),
    OdooRelease('9.saas~10e', '20/03/2016'),
    OdooRelease('9.saas~11e', '20/05/2016'),
    OdooRelease('9.saas~12e', '20/07/2016'),
    OdooRelease('10.0+e', '08/09/2016'),
    OdooRelease('10.0e', '08/09/2016'),
    OdooRelease('10.saas~14+e', '07/03/2017'),
    OdooRelease('10.saas~15+e', '08/05/2017'),
    OdooRelease('10.saas~16+e', '08/08/2017'),
    OdooRelease('11.0+e', '06/10/2017'),
    OdooRelease('11.0e', '06/10/2017'),
    OdooRelease('saas~11.1+e', '12/12/2017'),
    OdooRelease('saas~11.3+e', '13/08/2018'),
    OdooRelease('12.0+e', '03/09/2018'),
    OdooRelease('saas~12.3+e', '06/08/2019'),
    OdooRelease('13.0+e', '02/10/2019'),
    OdooRelease('14.0+e', '30/09/2020'),
    OdooRelease('15.0+e', '06/10/2021'),
    OdooRelease('saas~15.1+e', '03/12/2021'),
    OdooRelease('saas~15.2+e', '04/02/2022'),
    OdooRelease('16.0+e', '13/10/2022'),
    OdooRelease('saas~16.1+e', '14/02/2023'),
    OdooRelease('saas~16.2+e', '27/03/2023'),
    OdooRelease('saas~16.3+e', '05/06/2023'),
    OdooRelease('saas~16.4+e', '07/08/2023'),
    OdooRelease('17.0+e', '09/11/2023'),
    OdooRelease('saas~17.1+e', '16/01/2024'),
    OdooRelease('saas~17.2+e', '19/03/2024'),
]

FRENCH_FORMAT = '%d/%m/%Y'

_logger = logging.getLogger(__name__)

class FrenchCertificationReport(models.AbstractModel):
    _name = "report.openerp_french_certification.french_certification"
    _description = "PoS French Certification"

    @api.model
    def _get_certification_pdf(self, database, username, company_name, **kw):
        # some Windows builds have a double `e` in the series, e.g. 12.0e+e instead of 12.0+e
        candidates = [r for r in RELEASES if database.version.replace("e+e", "+e").startswith(r.version)]
        if not candidates:
            _logger.info('Cannot find release of %r (version=%s)', database, database.version)
            # same error message than in the controller.
            raise UserError("La combinaison du numéro de contrat et du nom de base de données est incorrecte ou vous n'avez pas de base de données compatible (>= 9.0+e) sur ce contrat")

        database_version = candidates[0]

        # We want to find the most relevant date at which the client *might* have started using certified stuff
        # we want to cover ourselves from client's actions done in the past
        # starting with ping for the production time
        # ending with the most elevant app installation time
        # Waiting for odo & qdp to decide whether we have the l10n_fr_certification infos whithin the ping

        # pings are desc ordered, and we want the most ancient
        start_use_date = database.ping_ids.filtered(lambda p: p.version == database.version)[-1].ping_time
        app_invoicing = database.app_ids.filtered(lambda app: app.name in ('account', 'account_invoicing'))
        if not app_invoicing:
            raise UserError(u"La Facturation/Comptabilité n'est pas installée sur la base de données %s." % database.db_name)
        app_invoicing = app_invoicing.sorted('first_date')[0]
        invoicing_date = app_invoicing.first_date
        contains_pos = database.app_ids.filtered(lambda app: app.name == 'point_of_sale')

        if contains_pos:
            contains_pos = contains_pos.sorted('first_date')[0]
            invoicing_date = contains_pos.first_date

        version_gte_13 = parse_version(database_version.version) >= parse_version('13.0+e')
        database_consolidated = {'version': database_version.version,
                                 'version_gte_13': version_gte_13,
                                 'contains_pos': contains_pos,
                                 'retail_date': database_version.date,
                                 'start_use_date': start_use_date.strftime(FRENCH_FORMAT),
                                 }

        data = {'company': self.env.ref('base.main_company').sudo(),
                'partner_name': username,
                'database': database_consolidated,
                'contract_nb': database.subscription_id.client_order_ref,
                'company_name': company_name,
                'date': datetime.now().strftime(FRENCH_FORMAT),
                'invoice_date': max([invoicing_date, start_use_date.date()]).strftime(FRENCH_FORMAT)
                }

        report_ref = 'openerp_french_certification.certification_report'
        report = self.env["ir.actions.report"]
        pdf = report.with_user(SUPERUSER_ID)._render_qweb_pdf(report_ref, data=data)[0]
        return pdf

    def _get_report_values(self, docids, data):
        return data

    @api.model
    def _get_attachment(self, database, username, company_name, **kw):
        filename = "cgi286-I3-%s-%s-%s.pdf" % (database.db_uuid, database.version, database.subscription_id.client_order_ref)
        certif = self.env['ir.attachment'].sudo().search([('name', "=", filename)])
        pdf = self._get_certification_pdf(database, username, company_name)
        if not certif:
            certif.sudo().create({'name': filename,
                                  'res_model': 'sale.order',
                                  'res_id': database.subscription_id.id,
                                  'store_fname': filename,
                                  'datas': base64.b64encode(pdf),
                                  'mimetype': 'application/pdf'})
        return pdf
